@@ -5,7 +5,7 @@ use std::ffi::{CStr, CString};
 use std::hash::{BuildHasher, Hash};
 use std::marker;
 
-use crate::de::{Decode, Decoder, PairDecoder, PairsDecoder, ReferenceVisitor, SequenceDecoder};
+use crate::de::{Decode, Decoder, PairDecoder, PairsDecoder, SequenceDecoder, ValueVisitor};
 use crate::en::{Encode, Encoder, PairEncoder, SequenceEncoder};
 use crate::error::Error;
 use crate::internal::size_hint;
@@ -30,7 +30,7 @@ impl<'de> Decode<'de> for String {
 
         struct Visitor<E>(marker::PhantomData<E>);
 
-        impl<'de, E> ReferenceVisitor<'de> for Visitor<E>
+        impl<'de, E> ValueVisitor<'de> for Visitor<E>
         where
             E: Error,
         {
@@ -43,6 +43,11 @@ impl<'de> Decode<'de> for String {
                 write!(f, "a string")
             }
 
+            #[inline]
+            fn visit_owned(self, value: String) -> Result<Self::Ok, Self::Error> {
+                Ok(value)
+            }
+
             #[inline]
             fn visit_borrowed(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
                 self.visit_any(string)
@@ -72,7 +77,7 @@ impl<'de> Decode<'de> for Box<str> {
     where
         D: Decoder<'de>,
     {
-        Ok(<&str>::decode(decoder)?.into())
+        Ok(String::decode(decoder)?.into())
     }
 }
 
@@ -96,7 +101,7 @@ impl<'de> Decode<'de> for Cow<'de, str> {
 
         struct Visitor<E>(marker::PhantomData<E>);
 
-        impl<'de, E> ReferenceVisitor<'de> for Visitor<E>
+        impl<'de, E> ValueVisitor<'de> for Visitor<E>
         where
             E: Error,
         {
@@ -109,6 +114,11 @@ impl<'de> Decode<'de> for Cow<'de, str> {
                 write!(f, "a string")
             }
 
+            #[inline]
+            fn visit_owned(self, string: String) -> Result<Self::Ok, Self::Error> {
+                Ok(Cow::Owned(string))
+            }
+
             #[inline]
             fn visit_borrowed(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
                 Ok(Cow::Borrowed(string))
